/**
@author Cesar Vasquez Ibaceta
@version 3.0 01-06-04
*/

import javax.swing.*;
import java.lang.String.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.Graphics2D.*;
import java.io.*;


/**
Esta clase, nos permite crear nuestro panel de dibujo, en donde pondremos
las figuras que dibujemos. Ademas, es la encargada de implementar el 
MouseListener y MouseMoveListener para estar atenta a cuando queramos dibujar.
*/
public class DrawPanel extends JPanel
{
   private ArrayList figArray;
   private String option;
   private Figuras figuraTemporal;
   private Point2D punto1;
   private Point2D punto2;
   private Point2D punto4;
   private Color color;
   private int colorCode;
   private double dx, dy, d1, d2, k, factor;
   private double alpha, gamma;

   /**
   Crea un objeto drawPanel en donde dibujaremos las figuras.
   Ademas, se encarga de inicializar todas las variables criticas para que la
   funcion paintComponent no reciba arreglos y punteros vacios.
   */
   public DrawPanel()
   {
      punto1 = null;
      punto2 = null;
      punto4 = null;
      figArray =  new ArrayList();
      figuraTemporal = null;
      option = "NULL";
      
      setBackground(Color.WHITE);
      addMouseMotionListener(new MouseMotionHandler());
      addMouseListener(new MouseHandler());
   }



   /**
   Recibe un llamado cuando es presionado algun boton. Ademas se usa para 
   recibir el nombre del boton presionado como parametro
   @param mensaje recibe el nombre de un boton en forma de string cuando es 
   presionado algun boton.
   */
   public void setOption(String mensaje)
   {
      option = mensaje;                //se recibe que boton fue apretado
      //aca se inicializan las variables criticas, cada vez que se 
      //presiona un boton, para evitar que el programa reciba punteros nulos
      d1 = 0;
      d2 = 0;
      dx = 0;
      dy = 0;
      alpha = 0;
      gamma = 0;                       
      factor = 1;
      k = 1;
      punto4 = null;
      if (option.equals("limpiar")) figArray.clear();   
      repaint();     
   }
   
   /**
   Cuando un radio button es presionado, se invoca a este metodo para 
   modificar el color actual.
   @param c Color con que se estan dibujando las nuevas figuras
   @param code codigo del color anterior. (se usa en el formato .fig)
   */
   public void setColor(Color c, int code)
   {
      color = c;
      colorCode = code;
   }
   
   /**
   Este metodo se invoca cuando es presionado el boton de <I>guardar</I>.<BR>
   Se encarga de crear un objeto <I>guardarFigura</I> e invocar metodos en l
   para guardar el dibujo actual.
   */
   public void saveOption()
   {
      //se capturan errores al intentar leer el archivo
      try
      {
         int i;
         GuardarFiguras guardar = new GuardarFiguras(figArray);
         File archivo = guardar.save();  //se guarda el archivo .eda
         if (archivo == null) return;  //se comprueba que el archivo .fig exista
         //se crea un FileWrite para escribir en el archivo
         FileWriter out = new FileWriter(archivo); 
         //informacion estandar para el archivo .fig
         out.write("#FIG 3.2\n");
         out.write("Landscape\n");
         out.write("Center\n");
         out.write("Metrics\n");
         out.write("Letter\n");
         out.write("100.00\n");
         out.write("Single\n");
         out.write("-2\n");
         out.write("60 2\n");
         //se escriben en el archivo cada una de las figuras que se han creado.
         for (i = 0; i < figArray.size(); i++)
         {
            Figuras temporal = (Figuras)figArray.get(i);
            temporal.printIni(out);
            out.write("0 100 0 -1 6.40 ");
            temporal.printFin(out);
         }
         out.close();   //se cierra el archivo
      } 
      catch (Exception e) 
      {
         System.out.print("Error: " + e);
         System.exit(1);
      }
   }

   /**
   Este metodo se invoca cuando es presionado el boton de <I>cargar</I>.<BR>
   Se encarga de crear un objeto <I>cargarFigura</I> e invocar metodos en l
   para cargar un archivo con extension .eda
   */
   public void loadOption()
   {
      CargarFiguras cargar = new CargarFiguras();
      ArrayList temp = new ArrayList();   
      temp = cargar.load();
      if (temp.size() != 0) //se revisa si el ArrayList devuelto es valido
      {
         figArray.clear();
         figArray = temp;
      }
      repaint();
   }


   /**
   Metodo encargado de pintar y repintar lo que dibujemos
   @param g objeto grafico que representa el panel en el que nos encontramos.
   */
   public void paintComponent(Graphics g)
   {
      super.paintComponent(g);
      Graphics2D g2 = (Graphics2D)g;
      int i;
      //aca se realizan las operaciones de mover, escalar y rotar, segun 
      //corresponda
      if (option.equals("mover")) 
         g2.translate(dx , dy);
      if (option.equals("escalar") && punto4  !=  null) 
         g2.scale(factor, factor);
      if (option.equals("rotar") && punto4  !=  null) 
         g2.rotate(alpha, punto4.getX(), punto4.getY());
      //se recorre el ArrayList de figuras y de dibujan una por una
      for (i = 0; i < figArray.size(); i++)
      {
         //se recupera un objeto figuras del ArrayList
         Figuras temporal = (Figuras)figArray.get(i);
         //se convierte un objeto "figura" en la figura que representa
         Shape tempunto2 = temporal.getFigura();
         //se recupera el color de la figura.
         g2.setColor(temporal.getColor());
         //se dibuja la figura con el color correspondiente
         g2.draw((Shape)tempunto2);
      }
      //se dibuja la figura actual
      g2.setColor(color);  
      if (figuraTemporal != null) g2.draw((Shape)figuraTemporal.getFigura());
   }


   /**
   Clase Listener del mouse para nuestro panel de dibujo, detecta alguna 
   acciones estaticas del mouse, como clicks y o numero de clicks etc.
   */
   private class MouseHandler extends MouseAdapter
   {
      /**
      Reacciona cuando el boton del mouse es presionado.
      @param event Entrega info sobre el evento. En nuestro caso nos interesa 
      el lugar en donde el mouse fue presionado
      */
      public void mousePressed(MouseEvent event)
      {
         double tempunto1, tempunto2;
         
         punto1 = event.getPoint();  //guarda el ultimo click que se hizo
         if (punto4 != null && option.equals("rotar")) //cuando se esta rotando.
         {
            tempunto1 = Math.abs(punto4.getX() - punto1.getX());
            tempunto2 = punto4.distance(punto1);
            gamma = Math.acos(tempunto1 / tempunto2);
            if (punto4.getY() > punto1.getY()) gamma = -gamma; 
         }
      }
      
      
      /**
      Reacciona cuando el boton del mouse realiza un click.
      @param event Entrega info sobre el evento. En nuestro caso nos interesa 
      el lugar en donde se produjo el <i>click</i>
      */
      public void mouseClicked(MouseEvent event)
      {
         punto4 = event.getPoint();  //guarda el ultimo click
      }

      /**
      Reacciona cuando el boton del mouse es soltado.
      @param event Entrega info sobre el evento. En este caso, nos interesa el 
      lugar en donde el boton del mouse fue soltado.
      */
      public void mouseReleased(MouseEvent event)
      {
         Figuras temporal = null; //se elimina la figura temporal anterior
         
         punto2 = event.getPoint();  //detecta el lugar donde se solto el mouse
         //realiza acciones dependiendo del ultimo boton que fue presionado
         if (option.equals("ELIPSE")) 
            temporal = new Elipse(punto1, punto2, color, colorCode);
         else if (option.equals("RECTANGULO")) 
            temporal = new Rectangulo(punto1, punto2, color, colorCode);
         else if (option.equals("LINEA")) 
            temporal = new Linea(punto1, punto2, color, colorCode);
         else if (option.equals("escalar")) k = factor; 
         else if (option.equals("rotar")) gamma = alpha;
         else if (option.equals("mover")) 
         {
            d1 = dx;
            d2 = dy;
         }
         //se agrega la figura figura temporal, si es que existe
         if (temporal != null) figArray.add(temporal);  
         figuraTemporal = null;
         repaint();
      }
   }



   /**
   Listener que detecta eventos activos de nuestro mouse, tales como: 
   movimiento, arrastre, etc. y produce alguna accion.
   */
   private class MouseMotionHandler extends MouseMotionAdapter
   {
      private Point2D punto3;

      /**
      Reacciona cuando el mouse es arrastardo, arrojando informacion sobre el 
      ultimo punto.
      @param event entrega informacion sobre el evento. En este caso nos 
      interesa saber cual fue el utlimo punto en que el mouse fue arrastrado.
      */
      public void mouseDragged(MouseEvent event)
      {
         punto3 = event.getPoint();
         //se realizan acciones mientras el mouse es arrastrado
         if (option.equals("ELIPSE")) 
            figuraTemporal = new Elipse(punto1, punto3, color, colorCode);
         else if (option.equals("RECTANGULO")) 
            figuraTemporal = new Rectangulo(punto1, punto3, color, colorCode);
         else if (option.equals("LINEA")) 
            figuraTemporal = new Linea(punto1, punto3, color, colorCode); 
         else if (option.equals("mover"))
         {
            dx = event.getX() - punto1.getX();
            dy = event.getY() - punto1.getY();
            dx = d1 + dx;
            dy = d2 + dy;
         } 
         else if (option.equals("escalar"))
         {
            if (punto4 != null)
               factor = k * (punto4.distance(punto1) / punto4.distance(punto3));
         }
         else if (option.equals("rotar"))
         {
            if (punto4 != null)
               alpha = -angulo(punto4, punto3);
         }
         repaint();  
      }
   }
   
   /**
   Funcion encargada de calcular el angulo de rotacion a partir de un punto y
   el arrastre (Pc). 
   @param puntoA primer punto, obtenido al hacer click.
   @param puntoC tercer punto, recogido a medida que el mouse es arrastrado.
   @return beta es el angulo de giro entre el primer punto y el "eje x", y el 
   ultimo punto.
   */
   private double angulo(Point2D puntoA, Point2D puntoC)
   {
      final int DELTA = 5;
      double a, b, c;
      double beta;
      double temp;
      Point2D puntoB;
      
      //se crea un punto temporal con la misma componente x de Pa
      puntoB = new Point2D.Double(puntoA.getX() + DELTA, puntoA.getY()); 
      //se calculan las distancias entre los tres puntos
      a = puntoA.distance(puntoB);
      b = puntoB.distance(puntoC);
      c = puntoC.distance(puntoA);
      if (a == 0 || c == 0) return 0; //comprobacion para evitar incongruencias
      //se usa el teorema del coseno.
      temp = (-Math.pow(b, 2) + Math.pow(a, 2) + Math.pow(c, 2)) / (2 * a * c);
      beta = Math.acos(temp) + gamma;
      if (puntoC.getY() < puntoA.getY()) return beta;
      else return -beta;
   }
}

