/**
@author Cesar Vasquez Ibaceta
@version 3.0 01-06-04
*/

import javax.swing.*;
import javax.swing.JButton;
import javax.swing.JFrame;
import java.lang.String.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.awt.Toolkit.*;
import java.awt.geom.*;
import java.awt.Graphics2D.*;



/**
Clase Principal, ella se encargara de crear nuestras ventanas y paneles.
*/
public class Editor
{
   /**
   Este constructor, crea un Frame en donde iran el resto de nuestros objetos.
   Ademas setea la opcion de "salir de la aplicacion al cerrar la ventana"
   @param args <I>no se utiliza.</I>
   */
   public static void main(String[] args)
   {
      MainFrame frame = new MainFrame();
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.show();
   }
}


/**
Clase para crear nuestra Ventana principal, ademas se encarga de definir el
tamao de nuestra ventana y su posicion en la pantalla.
*/
class MainFrame extends JFrame
{
   private MainPanel mPanel;
   private final int tres = 3;
   private final int cuatro = 4;

   /**
   Se encarga de crear nuestra Ventana Principal, ademas de agregarle un Panel
   Principal (MPanel). Tambien se encarga de darle ciertos rasgos a nuestra
   ventana como tamao y posicion dentro de la pantalla.
   */
   public MainFrame()
   {
      //leemos el tamo de la pantalla para poder
      Toolkit screen = Toolkit.getDefaultToolkit(); 
      //hacer nuestra ventana de un tamao proporcional.
      Dimension screenSize = screen.getScreenSize();  
      int scHeight = screenSize.height;
      int scWidth = screenSize.width;

      setTitle("JavaPaint");
      //tamao de la ventana
      setSize(tres * scWidth / cuatro, tres * scHeight / cuatro);     
      //pocision de la ventana
      setLocation(scWidth / (cuatro + cuatro), scHeight / (cuatro * tres));    
      GridBagConstraints gbc = new GridBagConstraints();
      gbc.fill = GridBagConstraints.BOTH;
      mPanel = new MainPanel();
      mPanel.gridBM(gbc);
      Container contentPane = getContentPane();

      contentPane.add(mPanel);         //agregamos nuestro panel principal.
   }
}


/**
Clase para crear nuestro Panel Principal. Se encarga de agregar los subpaneles
de botones y de dibujo.
*/
class MainPanel extends JPanel
{
   private ButtonPanel bActPanel;
   private ButtonPanel bFigPanel;
   private ButtonPanel bLSPanel;
   private ColorPanel cPanel;
   private DrawPanel dPanel;
   private ArrayList action;

   //fue lo mejor que encontre para poder pasar el CheckStyle ^_^
   private final int tres = 3;
   private final int dos = 2;
   private final int cuatro = 4;
   private final int cinco = 5;
   private final int cero = 0;
   private final int uno = 1;
   private final double upc = 1.0;
   private final double cpc = 0.0;
   /**
   Este constructor, crea un panel principal, y se encarga de crear cinco 
   subpaneles mas, llamando a las clases buttonPanel, colorPanel y drawPanel. 
   <BR>Tambien agrega botones a bActPanel, bLSPanel y bFigPanel segun el metodo 
   agregarBoton de la clase buttonPanel y agrega un ActionListener a dichos 
   botones.<BR>
   */
   public MainPanel()
   {  //use un action listener para los botones de accion y figuras.
      ActionListener pButton = new PressButton();
      //action listener para el boton de guardar
      ActionListener s = new SaveButton();
      //action listener para el boton de cargar
      ActionListener l = new LoadButton();   
      setLayout(new GridBagLayout());

      //se crean los cinco paneles mediantes los respectivos 
      //constructores de cada clase.
      bActPanel = new ButtonPanel(cuatro, uno, "Acciones");
      bFigPanel = new ButtonPanel(uno, tres, "Figuras");
      bLSPanel = new ButtonPanel(dos, uno, "Archivo");
      cPanel = new ColorPanel(cinco, uno, "Colores");
      dPanel = new DrawPanel();
      //aca se crean ColorActions para los RadioButtons de los colores
      ColorAction blk = new ColorAction(Color.BLACK, cero);
      ColorAction grn = new ColorAction(Color.GREEN, dos);
      ColorAction rd = new ColorAction(Color.RED, cuatro);
      ColorAction bl = new ColorAction(Color.BLUE, uno);
      ColorAction yw = new ColorAction(Color.YELLOW, dos * tres);
      //se invocan los metodos respectivos para agragar botones a cada
      //uno de los paneles.
      cPanel.addRadioButton("n e g r o", blk);
      cPanel.addRadioButton("v e r d e", grn);
      cPanel.addRadioButton("r o j o", rd);
      cPanel.addRadioButton("a z u l", bl);
      cPanel.addRadioButton("a m a r i l l o", yw);
      bActPanel.agregarBoton("escalar", pButton);
      bActPanel.agregarBoton("rotar", pButton);
      bActPanel.agregarBoton("mover", pButton);
      bActPanel.agregarBoton("limpiar", pButton);
      bFigPanel.agregarBoton("ELIPSE", pButton);
      bFigPanel.agregarBoton("RECTANGULO", pButton);
      bFigPanel.agregarBoton("LINEA", pButton);
      bLSPanel.agregarBoton("cargar", l);
      bLSPanel.agregarBoton("guardar", s);
   }

   /**
   metodo encargado de hacer la distribucion fisica de las componentes 
   dentro del panel principal.
   @param gbc GridBagConstraints al que se le aplicara el formato
   para la distribucion fisica de las componentes
   */
   public void gridBM(GridBagConstraints gbc)
   {
      //se utilizo GridBagLAyout para la distribucion fisica de los 
      //subpaneles dentro del panel principal
      gbc.gridx = cero;
      gbc.gridy = cero;
      add(cPanel, gbc);

      gbc.gridheight = tres;
      gbc.gridx = cero;
      gbc.gridy = uno;
      add(bActPanel, gbc);

      gbc.weighty = upc;
      gbc.weightx = upc;
      gbc.insets = new Insets(cinco, cinco, cinco, cinco);
      gbc.gridwidth = tres;
      gbc.gridheight = cuatro;
      gbc.gridx = uno;
      gbc.gridy = cero;
      add(dPanel, gbc);

      gbc.weighty = cpc;
      gbc.weightx = upc;
      gbc.gridwidth = tres;
      gbc.ipady = cuatro * tres;
      gbc.gridx = cero;
      gbc.gridy = cuatro;
      add(bFigPanel, gbc);

      gbc.weighty = cpc;
      gbc.weightx = cpc;
      gbc.gridwidth = uno;
      gbc.ipady = cuatro * tres;
      gbc.gridx = tres;
      gbc.gridy = cuatro;
      add(bLSPanel, gbc);
   }

   /**
   Clase interna, que implementa a la clase ActionListener.
   Es la encargada de detectar cuando alguien presiona un boton.
   */
   private class PressButton implements ActionListener
   {
      private String option;
      /**
      Detecta cuando se presiona un boton, y envia el nombre del boton a la
      clase drawPanel por medio del metodo <i><b>setOption</b></i>.
      @param event entrega el nombre del boton que fue presionado.
      */
      public void actionPerformed(ActionEvent event)
      {
         //cuando un boton es presionado, se llama al metodo
         option = event.getActionCommand();     
         dPanel.setOption(option);  //setOption para pasarle las variables que 
                                    //usara el panel de dibujo (dPanel)
      }
   }


   /**
   Clase Interna ColorAction que detecta que color se esta utilizando, y lo
   setea en el panel de dibujo.
   */
   private class ColorAction implements ActionListener
   {
      private Color bgcolor;
      private int colorCode;

      /**
      En este metodo se recibe el color y el codigo del color que se asigna
      al respectivo ColorAction y se guardan.
      @param c Color que se asigna al respectivo RadioButton
      @param code Codigo del color asignado. Se utilizara posteriormente 
      cuando se exporte a formato .fig
      */
      public ColorAction(Color c, int code)
      {
         bgcolor = c;
         colorCode = code;
      }

      /**
      Detecta cuando un radioButton es seleccionado.
      @param event <i>no se utiliza.</i>
      */
      public void actionPerformed(ActionEvent event)
      {
         dPanel.setColor(bgcolor, colorCode);
      }
   }

   /**
   Clase Interna para manejar el boton de guardar.
   */
   private class SaveButton implements ActionListener
   {
      /**
      Invoca al metodo saveOption de la clase drawPanel, para realizar la
      labor de guardado.
      @param event <i>no se utiliza.</i>
      */
      public void actionPerformed(ActionEvent event)
      {
         dPanel.saveOption();
      }
   }

   /**
   Clase Interna para manejar el boton de guardar.
   */
   private class LoadButton implements ActionListener
   {
      /**
      Invoca al metodo loadOption de la clase drawPanel, para realizar la
      labor de cargar.
      @param event <i>no se utiliza.</i>
      */
      public void actionPerformed(ActionEvent event)
      {
         dPanel.loadOption();
      }
   }
}

