/**
@author Cesar Vasquez Ibaceta
@version 3.0 01-06-04
*/

import javax.swing.filechooser.*;
import javax.swing.filechooser.FileFilter;
import java.util.*;
import java.io.File;

/**
Se encarga de Filtrar las extensiones desplegadas por el FileChooser,
para evitar el cargar un archivo no valido "por error".
*/
public class ExtensionsFileFilter extends FileFilter
{
   private ArrayList extensions = new ArrayList(); 
   private String description;
   
   /**
   Agrega extensiones al Filtro. Es la que se desplega en los tipos
   de archivo del FileChooser
   @param extension extension que sera agregada al Filtro
   */
   public void addExtension(String extension)
   {
      //si no comienza con un punto, se le agrega.
      if (!extension.startsWith("."))
         extension = "." + extension;
      extensions.add(extension.toLowerCase());  //se guardan en minusculas
   }
   
   /**
   Agrega la descripcion para las extensiones manejadas por este filtro.
   @param aDescription Descripcion para las extensiones.
   */
   public void setDescription(String aDescription)
   {
      description = aDescription;
   }
   
   /**
   Extrae la descripcion de este Filtro.
   @return Devulve la descripcion asociada a este Filtro.
   */
   public String getDescription()
   {
      return description;
   }  
   
   /**
   Comprueba si la extension del archivo concuerda con alguna de las extensiones
   que maneja este Filtro.
   @param f Archivo que se desa comparar con las extensiones.
   @return retorna verdadero, para mostrar los directorios y las extenciones de 
   este filtro. Y devuelve falso para las extensiones que no estan en este 
   filtro.
   */
   public boolean accept(File f)
   {
      if (f.isDirectory()) return true; //muestra los directorios tambien.
      //cambia en nombre del archivo a minusculas   
      String name = f.getName().toLowerCase();
      //lo compara con todas las extensiones
      for (int i = 0; i < extensions.size(); i++)  
         if (name.endsWith((String)extensions.get(i))) 
            return true;   //si alguna concuerda entrega true
      return false;     //si no, retorna false.
   }
}
