/**
@author Cesar Vasquez Ibaceta
@version 3.0 01-06-04
*/

import java.awt.Graphics2D.*;
import java.awt.geom.*;
import java.awt.*;
import java.io.*;
import javax.swing.*;

/**
Esta clase crea objetos del tipo <i>figuras</i>.
*/
public abstract class Figuras implements Serializable
{
   private Point2D punto1;
   private Point2D punto2;
   private Color color;
   private int colorCode;

   /**
   Este constructo recibe dos puntos, y el color que define una figura dentro de
   del editor de figuras.
   @param p1 primer punto que define la figura.
   @param p2 degundo punto que define la figura
   @param c color con que fue dibujada la figura
   @param code codigo del color con que fue dibujada la figura. Este codigo 
   se utilizara posteriormente en la exportacion del formato para xfig
   */
   Figuras(Point2D p1, Point2D p2, Color c, int code)
   {
      color = c;
      colorCode = code;
      punto1 = p1;
      punto2 = p2;
   }

   /**
   Este metodo se encarga de entregar el punto que se encuentra en la esquina 
   superior izquierda del rectangulo imaginario formado por los puntos p1 y p2.
   @return punto retorna el punto de la esquina superior izquierda.
   */
   protected Point2D direccion()
   {
      Point2D punto;
      double x;
      double y;
      double ancho;
      double alto;

      ancho = punto1.getX() - punto2.getX();
      alto = punto1.getY() - punto2.getY();

      if (ancho > 0) x = punto2.getX();
      else x = punto1.getX();

      if (alto > 0) y = punto2.getY();
      else y = punto1.getY();

      punto = new Point2D.Double(x, y);
      return punto;  //retorna el punto en la esquina superior izquierda
   }

   /**
   Metodo que retorna el color con fue dibujado esta figura.
   @return color asociado a esta figura.
   */
   public Color getColor()
   {
      return color;
   }
   
   /**
   Metodo implementado por las clases Elipse, Rectangulo y Linea   
   @return devuelve un objeto shape formado por los dos puntos del objeto figura
   */
   public abstract Shape getFigura();

   /**
   Metodo implementado por las clases Elipse, Rectangulo y Linea
   @param out FileWriter asociado al archivo .fig en donde se exportara
   la figura
   @throws IOException arroja una excepcion cuando hay problemas al
   escribir en archivo
   */
   public abstract void printIni(FileWriter out) throws IOException;

   /**
   Metodo implementado por las clases Elipse, Rectangulo y Linea
   @param out FileWriter asociado al archivo .fig en donde se exportara
   la figura
   @throws IOException arroja una excepcion cuando hay problemas al
   escribir en archivo
   */
   public abstract void printFin(FileWriter out) throws IOException;
}



/**
Crea un objeto Elipse con dos puntos y un color asociado a el.
Extiende a la clase figuras.
*/
class Elipse extends Figuras
{
   private Point2D punto1;
   private Point2D punto2;
   private int code;
   private double largo, alto;

   /**
   Este constructor recibe dos puntos, y el color que define una Elipse 
   dentro del editor de figuras.
   @param p1 primer punto que define la figura.
   @param p2 degundo punto que define la figura
   @param c color con que fue dibujada la figura
   @param code codigo del color con que fue dibujada la figura. Este codigo 
   se utilizara posteriormente en la exportacion del formato para xfig
   */
   Elipse(Point2D p1, Point2D p2, Color color, int colorCode)
   {
      super(p1, p2, color, colorCode);  
      punto1 = p1;
      punto2 = p2;
      code = colorCode;
   }

   /**
   Devuelve un objeto <b>Shape</b> que es una Ellipse2D. 
   @return devulve una Ellipse2D en forma de Shape.
   */
   public Shape getFigura()
   {
      //se obtiene el punto en la esquina superior izquierda
      Point2D punto = direccion();  
      largo = Math.abs(punto1.getX() - punto2.getX());    //diametro de ancho
      alto = Math.abs(punto1.getY() - punto2.getY());    //diametro de alto

      Ellipse2D forma = new Ellipse2D.Double(punto.getX(), 
         punto.getY(), largo, alto);
      return forma; 
   }

   /**
   Escribe en un FileWrite asociado a un File, informacion referente 
   al objeto, para crear el archivo .fig
   @param out Aca se escribe informacion especifica sobre este objeto.
   @thrwos IOException arroja una excepcion cuando hay problemas al 
   escribir en <b>out</out>
   */
   public void printIni(FileWriter out) throws IOException
   {
      //informacion para definir que es una Elipse
      out.write("1 1 0 1 " + code + " ");  
   }

   /**
   Escribe en un FileWrite asociado a un File, informacion referente 
   al objeto, para crear el archivo .fig
   @param out Aca se escribe informacion especifica sobre este objeto.
   @thrwos IOException arroja una excepcion cuando hay problemas al escribir 
   en <b>out</out>
   */
   public void printFin(FileWriter out) throws IOException
   {
      int cx, cy;
      
      Point2D punto = direccion();
      cx = (int)(punto.getX() + largo / 2);
      cy = (int)(punto.getY() + alto / 2);
      out.write("1 0.0 "); //informacion que el editor no maneja
      out.write(cx + " " + cy + " ");           //centro de la elipse
      out.write((int)(largo / 2) + " " + (int)(alto / 2) + " ");    //diametros
      out.write(cx + " " + cy + " ");           //punto inicial (el centro)
      //punto final (un borde)
      out.write(cx + " " + (int)(cx + largo / 2) + "\n");   
   }
}


/**
Crea un objeto Rectangulo con dos puntos y un color asociado a el.
Extiende a la clase figuras.
*/
class Rectangulo extends Figuras
{
   private Point2D punto1;
   private Point2D punto2;
   private int code;    //codigo del color para la exportacion
   private double largo, alto;

   /**
   Este constructor recibe dos puntos, y el color que define una figura 
   dentro del editor de figuras.
   @param p1 primer punto que define la figura.
   @param p2 degundo punto que define la figura
   @param c color con que fue dibujada la figura
   @param code codigo del color con que fue dibujada la figura. Este codigo 
   se utilizara posteriormente en la exportacion del formato para xfig
   */
   Rectangulo(Point2D p1, Point2D p2, Color color, int colorCode)
   {
      super(p1, p2, color, colorCode);  
      punto1 = p1;
      punto2 = p2;
      code = colorCode;
   }

   /**
   Devuelve un objeto <b>Shape</b> que es una Rectangle2D. 
   @return devulve una Rectangle2D en forma de Shape.
   */
   public Shape getFigura()
   {
      Point2D punto = direccion();
      largo = Math.abs(punto1.getX() - punto2.getX());    //largo del rectangulo
      alto = Math.abs(punto1.getY() - punto2.getY());    //alto del rectangulo

      Rectangle2D forma = new Rectangle2D.Double(punto.getX(),
         punto.getY(), largo, alto);
      return forma; 
   }

   /**
   Escribe en un FileWrite asociado a un File, informacion referente 
   al objeto, para crear el archivo .fig
   @param out Aca se escribe informacion especifica sobre este objeto.
   @thrwos IOException arroja una excepcion cuando hay problemas al escribir 
   en <b>out</out>
   */
   public void printIni(FileWriter out) throws IOException
   {
      //informacion que define a un rectangulo
      out.write("2 2 0 1 " + code + " "); 
   }

   /**
   Escribe en un FileWrite asociado a un File, informacion referente 
   al objeto, para crear el archivo .fig
   @param out Aca se escribe informacion especifica sobre este objeto.
   @thrwos IOException arroja una excepcion cuando hay problemas al escribir 
   en <b>out</out>
   */
   public void printFin(FileWriter out) throws IOException
   {
      int x1, y1, x2, y2;
      
      x1 = (int)punto1.getX();
      y1 = (int)punto1.getY();
      x2 = (int)punto2.getX();
      y2 = (int)punto2.getY();

      out.write("0 0 0 0 0 5\n");   //informacion que el editor no maneja
      //se comienzan a escribir los 5 puntos. Los cuatro vertices 
      //y al final se repite el ultimo
      out.write("\t" + x1 + " " + y1 + " ");
      out.write(x1 + " " + y2 + " ");
      out.write(x2 + " " + y2 + " ");
      out.write(x2 + " " + y1 + " ");
      out.write(x1 + " " + y1 + "\n");
   }
}



/**
Crea un objeto Linea con dos puntos y un color asociado a el.
Extiende a la clase figuras.
*/
class Linea extends Figuras
{
   private Point2D punto1;
   private Point2D punto2;
   private int code;

   /**
   Este constructor recibe dos puntos, y el color que define una figura 
   dentro del editor de figuras.
   @param p1 primer punto que define la figura.
   @param p2 degundo punto que define la figura
   @param c color con que fue dibujada la figura
   @param code codigo del color con que fue dibujada la figura. Este codigo 
   se utilizara posteriormente en la exportacion del formato para xfig
   */
   Linea(Point2D p1, Point2D p2, Color color, int colorCode)
   {
      super(p1, p2, color, colorCode);  
      punto1 = p1;
      punto2 = p2;
      code = colorCode;
   }

   /**
   Devuelve un objeto <b>Shape</b> que es una Line2D. 
   @return devulve una Line2D en forma de Shape.
   */
   public Shape getFigura()
   {
      //crea una linea con los dos puntos.
      Line2D forma = new Line2D.Double(punto1, punto2);
      return forma;
   }

   /**
   Escribe en un FileWrite asociado a un File, informacion referente 
   al objeto, para crear el archivo .fig
   @param out Aca se escribe informacion especifica sobre este objeto.
   @thrwos IOException arroja una excepcion cuando hay problemas al 
   escribir en <b>out</out>
   */
   public void printIni(FileWriter out) throws IOException
   {
      //informacion que define a una linea.
      out.write("2 1 0 1 " + code + " ");    
   }

   /**
   Escribe en un FileWrite asociado a un File, informacion referente 
   al objeto, para crear el archivo .fig
   @param out Aca se escribe informacion especifica sobre este objeto.
   @thrwos IOException arroja una excepcion cuando hay problemas al escribir 
   en <b>out</out>
   */
   public void printFin(FileWriter out) throws IOException
   {
      int x1, y1, x2, y2;
      
      x1 = (int)punto1.getX();
      y1 = (int)punto1.getY();
      x2 = (int)punto2.getX();
      y2 = (int)punto2.getY();

      out.write("0 0 12 0 0 2\n");  //informacion que el editor no maneja
      out.write("\t" + x1 + " " + y1 + " ");   //primer punto de la linea
      out.write(x2 + " " + y2 + "\n");       //segundo punto de la linea
   }
}
